# -!- coding: utf-8 -!-
import re, os, sys, requests, json, time, math, base64, traceback
from Crypto.Cipher import AES
from copy import deepcopy

class 起点计费系统api():
    AppKey = ''
    key = ''
    headers = {'Content-Type':'application/json', 'Connection': 'close'}
    审核字典 = {-1:-1, 0:0, 2:2, '全部':-1, True:2, False:0}
    启用字典 = {-1:-1, 0:0, 1:1, '全部':-1, True:1, False:0}
    # 1:百度桌面, 2:360PC, 3:百度移动, 4:360移动, 5:搜狗PC, 6:搜狗移动, 7:神马
    搜索引擎字典 = {1:1, 2:2, 3:3, 4:4, 5:5, 6:6, 7:7, -1:-1, '全部':-1}
    覆盖字典 = {0:0, 1:1, True:1, False:0}
    请求失败日志路径 = ''
    
    def __init__(self, appkey='', key=''):
        # 漏洞补丁
        self.headers = deepcopy(self.headers)
        self.审核字典 = deepcopy(self.审核字典)
        self.启用字典 = deepcopy(self.启用字典)
        self.搜索引擎字典 = deepcopy(self.搜索引擎字典)
        self.覆盖字典 = deepcopy(self.覆盖字典)

        if appkey: self.AppKey = appkey
        if key: self.key = key
    
    def aes_ecb_pkcs7(self, req):
        req = req + (16- len(req) % 16) * chr(16 - len(req) % 16)
        req = str.encode(req)
        aes = AES.new(str.encode(self.key), AES.MODE_ECB)
        encrypted_text = str(base64.encodebytes(aes.encrypt(req)), encoding='utf8').replace('\n', '')
        decrypted_text = str(aes.decrypt(base64.decodebytes(bytes(encrypted_text, encoding='utf8'))).rstrip(b'\0').decode("utf8"))
        return encrypted_text
    
    def 请求服务器(self, url, datas):
        入参data = datas
        datas = {"AppKey": self.AppKey, "TimeStamp": math.floor(time.time()), "Data": datas}
        req = f'"{self.aes_ecb_pkcs7(json.dumps(datas))}"'
        for n in range(10):  # 因为计费系统的服务器太垃圾, 经常会无响应, 所以需要多次重试
            try:
                return requests.post(url, headers=self.headers, data=req).json()
            except Exception as err:
                print(err)
                e = err
                if 'Max retries exceeded with url' in str(err): time.sleep(1)
                # 服务器无响应时, request库不断进行重新请求, 于是便会导致连接过多
        if self.请求失败日志路径:
            try:
                with open(self.请求失败日志路径, 'r', encoding='utf8') as fa: text = fa.read()
            except: text = ''
            当前时间 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
            text = f"{text}\n\n{当前时间} - {入参data}"
            with open(self.请求失败日志路径, 'w+', encoding='utf8') as fa: fa.write(text)
        raise e

    def 获取关键词(self, 审核=True, 启用=True, 搜索引擎='全部'):  # 返回结构: [{}, {}]
        allkw = []
        TotalPageCount = ''
        for page in range(1, 10000):
            print(f"\r正在获取关键词 第{page}/{TotalPageCount}页", end='    ')
            审核, 启用, 搜索引擎 = self.审核字典[审核], self.启用字典[启用], self.搜索引擎字典[搜索引擎]
            req = {"AuditState":审核, "RankType":搜索引擎, "PageSize":2000, "PageNum":page, 'IsEnabled':启用}
            r = self.请求服务器('http://api.235kk.com/api/Words', req)
            message, data, state = [r[x] for x in ('message', 'data', 'state')]
            TotalNum, Items, CurrentPage, TotalPageCount, OtherData = [data[x] for x in ('TotalNum', 'Items', 'CurrentPage', 'TotalPageCount', 'OtherData')]
            for x in Items:
                x['kwid'] = x.pop('WordID')
                x['最新排名'] = x.pop('RankNum') or 51
                allkw.append(x)
            if page >= TotalPageCount:
                print(f"\n提取关键词完成, 共{len(allkw)}条.")
                return allkw

    def 修改审核状态(self, kws):  # 传入结构: [{'kwid':kwid, '审核':True}, {'kwid':kwid, '审核':True}]
        kws = [dict(WordID=kw['kwid'], AuditState=self.审核字典[kw['审核']]) for kw in kws]
        r = self.请求服务器('http://api.235kk.com/api/WordsInfo', kws)
        print('\n修改审核状态  '+r['message'], '  请求状态:',r['state'], '  修改成功数量:',r['data'])
        return r['data']
    
    def 修改启用状态(self, kws):  # 传入结构 [{'kwid':kwid, '启用':True, '说明':说明}]
        kws = [dict(WordID=k['kwid'], State=self.启用字典[k['启用']], Remark=k['说明']) for k in kws]
        r = self.请求服务器('http://api.235kk.com/api/WordsEnable', kws)
        print('\n关键词禁用接口  ' + r['message'], '  请求状态:', r['state'], '  修改成功数量:', r['data'])
    
    def 提交排名(self, kws):  # 传入结构 [{'kwid':kwid, '排名':排名, '时间':时间, '覆盖':False}]
        kws = [dict(WordID=k['kwid'], RankNum=k['排名'], RankTime=k['时间'], IsOverride=self.覆盖字典[k.get('覆盖', False)]) for k in kws]
        r = self.请求服务器('http://api.235kk.com/api/WordsRank', kws)
        print('\n修改排名  '+r['message'], '  请求状态:', r['state'], '  修改成功数量:', r['data'])
        r = r['data']
        return r

    def 根据关键词查询kwid(self, kws, 关键词字段='kw', 域名字段='domain', kwid字段='WordID'):
        allkw = self.获取关键词(审核='全部', 启用='全部', 搜索引擎='全部')
        allkw = {(k['Name'], k['DomainName']):k['WordID'] for k in allkw}
        for k in kws:
            k[kwid字段] = allkw.get((k[关键词字段], k[域名字段]))
        return kws



# -!- coding: utf-8 -!-
from copy import deepcopy

class dfa():
    # 无意义词库,在检测中会跳过
    无意义词库 = {' ', '&', '!', '！', '@', '#', '$', '￥', '*', '^', '%', '?', '？', '<', '>', "《", '》'}
    词库 = {}

    def __init__(self, 敏感词库):
        # 漏洞补丁
        self.无意义词库 = deepcopy(self.无意义词库)
        self.词库 = deepcopy(self.词库)
        
        for kw in 敏感词库: self.添加词库(kw)  # 添加词库

    def 添加词库(self, word):
        词库 = self.词库
        for kw in word:
            词库 = 词库.setdefault(kw, {'is_end': False})
        词库['is_end'] = True

    def 从指定的索引开始检测敏感词(self, text, 起始索引, 匹配模式='最长'):
        找到的索引 = []
        rlong = 0
        敏感词长度 = 0  # 包括特殊字符的敏感词的长度
        词库 = self.词库
        for i in range(起始索引, len(text)):
            word = text[i]
            if word in 词库:
                敏感词长度 += 1
                词库 = 词库[word]
                if 词库["is_end"]:
                    if 匹配模式 == '最长':
                        rlong = 敏感词长度
                    elif 匹配模式 == '最短':
                        return [敏感词长度]
                    elif 匹配模式 == '全部':
                        找到的索引.append(敏感词长度)
                    else:
                        raise Exception("匹配模式错误!")
            elif word in self.无意义词库:
                敏感词长度 += 1
            else:
                break
        return [rlong] if rlong else 找到的索引


    def 检测敏感词(self, text, 匹配模式='最长'):
        相匹配的词 = []
        for i in range(len(text)):
            找到的索引 = self.从指定的索引开始检测敏感词(text, i, 匹配模式=匹配模式)
            for x in 找到的索引: 相匹配的词.append(text[i: i + x])
        return 相匹配的词

if __name__ == '__main__':
    print(dfa(敏感词库=["七", "七分彩", '七分彩在线博赌']).检测敏感词('七分彩在线博赌', 匹配模式='最短'))
    print(dfa(敏感词库=["七", "七分彩", '七分彩在线博赌']).检测敏感词('七分彩在线博赌', 匹配模式='最长'))
    print(dfa(敏感词库=["七", "七分彩", '七分彩在线博赌']).检测敏感词('七分彩在线博赌', 匹配模式='全部'))
    # {'七'}
    # {'七分彩在线博赌'}
    # {'七分彩在线博赌', '七', '七分彩'}



# -!- coding: utf-8 -!-
import time, json, pymysql
import pandas as pd
from math import ceil


class 过滤器():

    def __init__(self, name=None, 条件=None, orm=None, 查询字段='*'):
        self.name = name
        self.条件 = 条件
        self.orm = orm
        self.c = orm.c
        self.查询字段 = 查询字段

    def copy(self, 条件, 查询字段=None):
        return 过滤器(name=self.name, 条件=条件, orm=self.orm, 查询字段=查询字段 or self.查询字段)

    def __eq__(self, o):  # ==
        if o is None:
            return self.copy(f"{self.name} is null")
        return self.copy(f"{self.name} = {json.dumps(o, ensure_ascii=False)}")

    def __ne__(self, o):  # !=
        if o is None:
            return self.copy(f"{self.name} is not null")
        return self.copy(f"{self.name} != {json.dumps(o, ensure_ascii=False)}")

    def __lt__(self, o):  # <
        return self.copy(f"{self.name} < {json.dumps(o, ensure_ascii=False)}")

    def __le__(self, o):  # <=
        return self.copy(f"{self.name} <= {json.dumps(o, ensure_ascii=False)}")

    def __gt__(self, o):  # >
        return self.copy(f"{self.name} > {json.dumps(o, ensure_ascii=False)}")

    def __ge__(self, o):  # >=
        return self.copy(f"{self.name} >= {json.dumps(o, ensure_ascii=False)}")

    def __and__(self, o):  # &
        if self.条件 == o.条件: return self.copy(self.条件)
        return self.copy(f"({self.条件}) and ({o.条件})")

    def __or__(self, o):  # |
        if self.条件 == o.条件: return self.copy(self.条件)
        return self.copy(f"({self.条件}) or ({o.条件})")

    def __invert__(self):  # ~
        return self.copy(f"not ({self.条件})")

    def re_m(self, o=''):
        if o: return self.copy(f"{self.name} regexp {json.dumps(o, ensure_ascii=False)}")
        raise Exception("正则内容不能为空")
    
    def __getitem__(self, key):
        type_ = type(key)
        if type_ is slice:
            start, stop, step = key.start or 0, key.stop or 0, key.step
            if (start < 0) or (stop < 0): raise Exception("不支持负数切片")
            elif start and stop: limit = f" limit {start}, {stop - start}"
            elif start: limit = f" limit {start}, 9999999999999999"
            elif stop: limit = f" limit 0, {stop}"
            else: limit = ""
            sql = f"select {self.查询字段}" + " from {数据表}" + (f" where {self.条件}" if self.条件 else '') + limit
            return self.orm.sql(sql)
        elif type_ is int:  # [3]
            if key < 0: raise Exception("不支持负数切片")
            sql = f"select {self.查询字段}" + " from {数据表}" + (f" where {self.条件}" if self.条件 else '') + f" limit {key}, 1"
            return self.orm.sql(sql)[0]
        elif type_ is str:  # 查询单个字段
            return self.copy(self.条件, key)
        elif type_ is dict:  # 字段重命名
            查询字段 = ','.join(f"{k} as {v}" for k, v in key.items())
            return self.copy(self.条件, 查询字段)
        elif type_ is tuple:  # 部分字段重命名
            查询字段 = []
            for x in key:
                if type(x) is str: 查询字段.append(x)
                else:
                    for k, v in x.items(): 查询字段.append(f"{k} as {v}")
            查询字段 = ','.join(查询字段)
            return self.copy(self.条件, 查询字段)
        else:  # orm[过滤器]
            if self.条件:
                return self.copy(f"({self.条件}) and ({key.条件})", 查询字段=self.查询字段)
            return self.copy(key.条件, 查询字段=self.查询字段)

    def __setitem__(self, key, value):
        type_ = type(key)
        if type_ is slice: raise Exception("不支持切片修改, 仅支持条件修改")
        elif type_ is int: raise Exception("不支持索引修改, 仅支持条件修改")
        elif type_ is str:
            zds = {key}
            条件 = f" where {self.条件}" if self.条件 else ''
        elif type_ is tuple:
            zds = set(key)
            条件 = f" where {self.条件}" if self.条件 else ''
        else:
            if self.查询字段 == '*':
                zds = set(self.orm.columns)
                pk = self.orm.pk
                if (pk not in value) and (pk in zds): zds.remove(pk)
            else:
                zds = set(self.查询字段.split(','))
            if self.条件:
                条件 = f" where ({self.条件}) and ({key.条件})"
            else:
                条件 = f" where {key.条件}"
        value = {k: value.get(k) for k in zds}
        value = ','.join([f"{k}={json.dumps(v, ensure_ascii=False)}" for k, v in value.items()])
        sql = "update {数据表}" + f" set {value}" + 条件
        return self.orm.sql(sql)

    def delete(self):
        sql = "delete from {数据表}" + f" where {self.条件}"
        return self.orm.sql(sql=sql)

    def isin(self, o=[]):
        if not o: return self.copy(f"({self.name} is null) and ({self.name} is not null)")
        o = set(o)
        条件null = None
        if None in o:
            条件null = f"{self.name} is null"
            o.remove(None)
            if not o: return self.copy(条件null)
        types = set(type(x) for x in o)
        if (types == {str}) or (types == {int}):
            条件in = f"{self.name} in ({','.join(json.dumps(x, ensure_ascii=False) for x in o)})"
            if 条件null: return self.copy(f"({条件null}) or ({条件in})")
            return self.copy(条件in)
        raise Exception("isin列表只接收[None, int] 或 [None, str]")

    def notin(self, o=[]):
        o = set(o)
        if not o: return self.copy(f"({self.name} is null) or ({self.name} is not null)")
        elif None in o:
            o.remove(None)
            条件null = f"{self.name} is not null"
            if o:
                types = set(type(x) for x in o)
                if (types != {str}) and (types != {int}): raise Exception("notin列表只接收[None, int] 或 [None, str]")
                条件in = f"{self.name} not in ({','.join(json.dumps(x, ensure_ascii=False) for x in o)})"
                return self.copy(f"({条件null}) and ({条件in})")
            return self.copy(条件null)
        else:
            types = set(type(x) for x in o)
            if (types != {str}) and (types != {int}): raise Exception("notin列表只接收[None, int] 或 [None, str]")
            条件in = f"{self.name} not in ({','.join(json.dumps(x, ensure_ascii=False) for x in o)})"
            条件null = f"{self.name} is null"
            return self.copy(f"({条件null}) or ({条件in})")

class cc():
    pass

class orm_mysql():

    '''

<<使用教程>>


一, 连接数据库


账号 = {'host': '', 'user': '', 'passwd': '', 'port': xxxx, 'db':'', 'sheet':''}
orm = orm_mysql(账号=账号, 持久连接=True)
持久连接=True时, 每次增删改查的速度比 持久连接=False 快0.5秒, 但用完后需要手动调用 self.close()
持久连接=False时, 每次执行增删改查, 都会创建一个临时的mysql连接, 用完后立即自动关闭, 所以需要多0.5秒


二, 添加数据


kw = {'title':xxx, 'content':xxx}

orm.append(kw)  # 添加一条数据

orm.数据分组尺寸 = 5000  # 默认为1000, 数值越大上传越快, 但mysql本身不支持无限大的分组尺寸
orm += [kw1, kw2, kw3]  # 批量添加数据


三, 过滤器[ ]


1.通过过滤器[ ]查询

查询所有数据: orm[:]

查询 id>=10 的数据: orm[orm.c.id >= 10][:]

查询 id==10 的数据: orm[orm.c.id == 10][:]

查询 title=='t' 的数据: orm[orm.c.title == 't'][:]

用正则表达式查询: orm[orm.c.title.re_m('.*')][:]

查询id在(1,3,5)里面的数据: orm[orm.c.id.isin([1,3,5])][:]

查询title在('t1','t2')里面的数据: orm[orm.c.title.isin(['t1','t2'])][:]

查询id不在(1,3,5)里面的数据: orm[orm.c.id.notin([1,3,5])][:]

查询title不在('t1','t2')里面的数据: orm[orm.c.title.notin(['t1','t2'])][:]

并联查询:
orm[(orm.c.id > 5) & (orm.c.title.re_m('.*'))][:]  # and
orm[(orm.c.id > 5) | (orm.c.title.re_m('.*'))][:]  # or
orm[~ (orm.c.id > 5)]  # 将条件取反
orm[() & (~ (() | (() & ()) | ()))][:]  # 可无限嵌套

串联查询
orm[orm.c.id > 5][orm.c.title == 't'][][][]...[:]  # 可无限嵌套

并联和串联混合查询
orm[并联语句][并联语句][并联语句][并联语句]...[:]

# 只查询某几列
orm[一系列过滤器]['title'][:]  # 只查询title字段
orm[一系列过滤器]['title', 'id'][:]  # 只查询title和id字段
orm[一系列过滤器]['title', {'id':'pk'}][:]  # 只查询title和id字段, 并且把id重命名为pk

# 切片
查询全部: orm[一系列过滤器][:]
查询前100条: orm[一系列过滤器][:100]
查询第101条开始的: orm[一系列过滤器][100:]
查询第100条: orm[一系列过滤器][100]


2.通过过滤器[ ]删除数据

orm[一系列过滤器].delete()
orm[orm.c.id >= 10].delete()


3.通过过滤器[ ]修改数据

kw = dict(title=xx, content=xx, 相似度=xx, ....)
orm[一系列过滤器] = kw  # 对所有字段赋值
orm[一系列过滤器]['title'] = kw  # 仅对title字段赋值
orm[一系列过滤器]['title','content'] = kw  # 仅对title和content字段赋值
'''

    数据分组尺寸 = 1000
    添加失败立即报错 = True
    极致上传 = False

    def __init__(self, 账号, 持久连接=True):
        self.添加失败的数据 = []  # 1
        self.提取账号(账号)  # 1
        self.持久连接 = 持久连接  # 1
        if 持久连接: self.conn, self.cursor = self.创建连接()  # 2
        self.获取字段列表()  # 3  self.columns, self.pk

        # 4 创建过滤器
        self.c = c = cc()
        for name in self.columns: exec(f"c.{name} = 过滤器(name='{name}', orm=self)")
        self.过滤器 = eval(f"c.{self.columns[-1]}")

    def 创建连接(self):
        for i in range(5):
            try:
                conn = pymysql.connect(**self.账号)
                cursor = conn.cursor(cursor=pymysql.cursors.DictCursor)
                return conn, cursor
            except Exception as err:
                e = err
                time.sleep(1)
        raise e

    def 提取账号(self, o):
        def get账号(o):
            r = {}
            for k, v in o.items():
                if type(v) is dict: r.update(get账号(v))
                else: r[k] = v
            return r
        o = get账号(o)
        self.账号 = dict(host=o['host'], user=o['user'], passwd=o['passwd'], port=o['port'], db=o['db'])
        self.name = o['sheet']

    def __getitem__(self, key): return self.过滤器.__getitem__(key)

    def __setitem__(self, key, value): return self.过滤器.__setitem__(key, value)

    def close(self): return self.conn.close()

    def sql(self, sql=None, **kvs):
        数据类型 = kvs.get('数据类型', 'list').lower()
        if self.持久连接:
            conn, cursor = self.conn, self.cursor
            conn.ping(reconnect=True)
        else:
            conn, cursor = self.创建连接()
        if sql:
            sql = sql.replace('{数据表}', self.name)
            cursor.execute(sql)
            datas = list(cursor.fetchall())
            conn.commit()
            if not self.持久连接: conn.close()
            if 数据类型 in ('pandas', 'df'): return pd.DataFrame(datas)
            return datas
        return conn, cursor, self.name

    def append(self, kw): return self.添加数据(datas=kw)
    
    def __iadd__(self, kws):  # 同list: orm += [{'title': '2020-02-25-0'}, {'title': '2020-02-25-1'}]
        r = self.添加数据(datas=kws)
        print(r)
        return self

    def 获取字段列表(self):
        sql = "select column_name as name,column_key as ispk from information_schema.columns where table_name = '{数据表}'"
        self.columns = columns = []
        self.pk = ''
        for x in self.sql(sql=sql):
            columns.append(x['name'])
            if x['ispk']: self.pk = x['name']

    def 添加数据(self, datas):
        conn, cursor, 数据表 = self.sql()
        if type(datas) is dict:
            try:
                字段s = datas.keys()
                sql = f"insert into {数据表}({', '.join(字段s)}) VALUES ({', '.join(('%s',)*len(字段s))})"
                cursor.execute(sql, tuple(datas.values()))
                conn.commit()
            except Exception as err:
                print(f"{err} - {datas}"[:20])
                self.添加失败的数据.append((datas, err))
                if not self.持久连接:
                    try: conn.close()
                    except: pass
                if self.添加失败立即报错:
                    raise Exception("存在添加失败的数据, 通过 orm.添加失败的数据 查看")
                return False
            if not self.持久连接:
                try: conn.close()
                except: pass
            return True
        else:
            size = self.数据分组尺寸
            fails = []
            字段s = set(y for x in datas for y in x.keys())
            datas = [tuple([kw.get(k) for k in 字段s]) for kw in datas]
            # 初始化数据库连接
            sql = f"insert into {数据表}({', '.join(字段s)}) VALUES ({', '.join(('%s',)*len(字段s))})"
            # 批量上传
            datas = [datas[size*(i-1): size*i] for i in range(1, ceil(len(datas)/size)+1)]
            group_number = len(datas)
            for i, idatas in enumerate(datas):
                print(f'\r正在批量上传第 {i+1}/{group_number} 组, 每组 {size} 条', end='    ')
                try:
                    cursor.executemany(sql, tuple(idatas))
                    conn.commit()
                except Exception as err:
                    conn.rollback()  # 回滚
                    for x in idatas: fails.append((x, err))
                    print(f'本组插入失败, 已回滚 - {err}')
            # 逐条上传
            if self.极致上传 and fails:
                print('\n已开启极致上传模式, 开始对批量上传失败的进行逐条上传')
                fails, datas, total = [], [a for a, b in fails], len(fails)
                for count, data in enumerate(datas):
                    print(f'\r正在逐条上传第 {count+1}/{total} 条', end='    ')
                    try: cursor.execute(sql, data)
                    except Exception as err:
                        fails.append((data, err))
                        print(f"{err} - {data}"[:20])
                    if (count+1) % size == 0: conn.commit()
                conn.commit()
            # 关闭数据库连接
            try:
                if not self.持久连接: conn.close()
            except: pass
            print(f'\n\n上传完成, 上传失败共 {len(fails)} 条')
            if fails:
                添加失败 = self.添加失败的数据
                for kw, err in fails: 添加失败.append((dict(zip(字段s, kw)), err))
                if self.添加失败立即报错: raise Exception("存在添加失败的数据, 通过 orm.添加失败的数据 查看")
                return False
            return True



import requests, time
from copy import deepcopy


class 树枝查百度pc排名():
    headers = {'apikey':''} 
    
    def __init__(self, apikey=''):
        self.headers = deepcopy(self.headers)
        if apikey: self.headers['apikey'] = apikey
        self.tids = {}

    def 推送并存储(self, kw, 关键词字段='kw', 域名字段='domain'):
        tid = self.push_kw(kw[关键词字段], kw[域名字段])
        self.tids[tid] = kw

    def push_kw(self, kw, domain):
        api_url = f'http://query.shuzhikj.com/api/platform/add_task?name={kw}&url={domain}&type=1&urlmode=1'
        r = requests.get(api_url, headers=self.headers)
        return str(int(r.json()['data']))
    
    def tid_get_rank(self, tid):
        api_url = f'http://query.shuzhikj.com/api/platform/get_taskrank?id={tid}'
        rank = requests.get(api_url, headers=self.headers).json()
        if type(rank['data']) is int: return rank['data']
        if rank['msg'] == '查询中': return None
        raise Exception(str(rank['msg']))
    
    def kw_get_rank(self, kw, domain, 最高次数=360, 等待间隔=10):  # 默认1小时
        等待间隔 = 等待间隔 or 0
        tid = self.push_kw(kw, domain)
        i = 0
        while (not 最高次数) or (i < 最高次数):
            i += 1
            rank = self.tid_get_rank(tid)
            if type(rank) is int: return rank
            if (not 最高次数) or (i < 最高次数):
                time.sleep(等待间隔)
        raise Exception("查询失败!")



# -!- coding: utf-8 -!-
import re, os, time, random, asyncio
from math import ceil
from random import shuffle
from multiprocessing.dummy import Pool as ThreadPool
# 同文件夹模块
from .tool import tool


def 异步简用(func):
    def newfunc(*vs, **kvs):
        async def main(*vs, **kvs):
            return await func(*vs, **kvs)
        return asyncio.run(main(*vs, **kvs))
    return newfunc


class 装饰器():
    tool = tool

    def 失败重试(最高次数=5, 重试间隔=0, 失败返回=Exception("-"), 装饰对象是实例方法=False):
        def 装饰器(func):
            最高次数_, 重试间隔_, 失败返回_ = 最高次数, 重试间隔, 失败返回
            name = str(func)
            name = re.findall('function (.+) at', name) or re.findall('method (.+) of', name) or [name]
            name = name[0]
            def newfunc(*vs, **kvs):
                最高次数 = kvs.pop('最高次数', 最高次数_)
                重试间隔 = max(0, kvs.pop('重试间隔', 重试间隔_))
                失败返回 = kvs.pop('失败返回', 失败返回_)
                for i in range(1, 最高次数 + 1):
                    try:
                        r = func(*vs, **kvs)
                        状态 = True
                        break
                    except Exception as err:
                        状态 = False
                        e = err
                        if i == 1: print(f"执行函数 {name} 第{i}/{最高次数}次失败, {重试间隔}秒后将进行重试    ", end='')
                        else: print(f"\r执行函数 {name} 第{i}/{最高次数}次失败, {重试间隔}秒后将进行重试    ", end='')
                        if i < 最高次数: time.sleep(重试间隔)
                if 状态:
                    if i > 1: print()
                    return r
                else: print(f"\n{e}")
                if type(失败返回) is Exception: raise e
                elif re.findall('method|function', str(type(失败返回))):
                    if 装饰对象是实例方法: vs = vs[1:]
                    return 失败返回(err=e, vs=vs, kvs=kvs)
                else: return 失败返回
            return newfunc
        return 装饰器

    @classmethod
    def 计时(cls, func):
        任务名称 = cls.tool.提取任务名称(func)
        def newfunc(*v, **kv):
            开始时间 = time.time()
            r = func(*v, **kv)
            print(f"{任务名称} - 运行了{round(time.time() - 开始时间, 2)}秒")
            return r
        return newfunc

    @classmethod
    def 多线程(cls, 更新代理=None, 更新ua=None, 判断是否执行成功=None):

        def 装饰器(func):

            寄存器 = dict(
                更新代理 = 更新代理 or False,
                判断是否执行成功 = 判断是否执行成功 or (lambda o: True),
                更新ua = 更新ua or (lambda : None)
            )
            
            def newfunc(*vs, datas={}, 线程数=20, 返值分组尺寸=None,
                最高次数=10, 重试间隔=0,
                更新代理=None, 代理有效时长=0, 更新ua=None, 判断是否执行成功=None, **kvs):

                # 自动修正返值分组尺寸
                if 返值分组尺寸: 返值分组尺寸 = min(返值分组尺寸, len(datas))

                # 确定 更新代理 的方法
                # False 表示不需要更新代理, 将关闭此功能
                # 如果不是False, 且未传入, 则使用预定义的方法
                if 更新代理 is not False:
                    更新代理 = 更新代理 or 寄存器['更新代理']

                # 确定 更新ua 的方法
                if 更新ua is False:  # False 表示不需要更新ua, 将关闭此功能
                    更新ua = lambda : None
                elif not 更新ua:  # 如果不是False, 且未传入, 则使用预定义的方法
                    更新ua = 寄存器['更新ua']
                
                # 确定 判断是否执行成功 的方法
                if 判断是否执行成功 is False:  # False 表示不需要判断, 直接返回成功
                    判断是否执行成功 = lambda o: True
                elif not 判断是否执行成功:  # 如果不是False, 且未传入, 则使用预定义的方法
                    判断是否执行成功 = 寄存器['判断是否执行成功']
                
                # 转化格式为: [{'key': 0, 'data': data0}, {'key': 1, 'data': data1}]
                if type(datas) is list:  # [{}, {}, ...]
                    datas = [dict(key=key, data=data) for key, data in zip(range(len(datas)), datas)]
                elif type(datas) is dict:  # {key1:{}, key2:{}, ...}
                    datas = [dict(key=key, data=data) for key, data in datas.items()]
                else:
                    raise Exception("datas格式须为: [{}, {}, ...]")

                def f2(kw):
                    try:
                        kw['return'] = func(*vs, **kvs, **kw['data'])
                        # **kw['data'] 放在 **kvs 后面, 估计可以用特别值覆盖通用值
                        kw['msg'] = 判断是否执行成功(kw)
                    except Exception as err:
                        kw['msg'] = err
                    return kw
            
                # 初始化数据
                成功 = []
                失败 = datas
                datas = None  # 释放内存
                random.shuffle(失败)
                if 更新代理:
                    更新代理()
                    cutime = time.time()
                    代理未失效 = True
                print()

                # 开始多线程查询
                for i次数 in range(1, 最高次数+1):
                    if not 失败:  # 如果查询完了, 就提前结束
                        break
                    else:
                        尚未查询 = cls.tool.数据分组(失败, 线程数)
                        失败 = []
                        total = len(尚未查询)
                        for count, kws in enumerate(尚未查询):
                            print(f'\r第{i次数}/{最高次数}次, 第{count + 1}/{total}组 {cls.tool.当前时间()}', end='    ')
                            更新ua()
                            if 更新代理:
                                if 代理未失效 is False:
                                    print('\r认为代理已失效, 正在自动更新代理', end='    ')
                                    更新代理()
                                    cutime = time.time()
                                elif 代理有效时长 and (time.time() - cutime >= 代理有效时长):
                                    print(f'\r代理已使用超过理论有效时长 {代理有效时长} 秒, 正在自动更新代理', end='    ')
                                    更新代理()
                                    cutime = time.time()
                            # 多线程请求
                            if 线程数 > 1:
                                pool = ThreadPool(线程数 + 2)
                                results = pool.map(f2, kws)
                                pool.close()
                                pool.join()
                            else:  # 单线程不需要使用pool
                                results = [f2(kws[0])]
                            # 处理请求结果
                            if 更新代理:
                                代理未失效 = False
                                for kw in results:
                                    if kw['msg'] is True:
                                        代理未失效 = True
                                        成功.append(kw)
                                    else:
                                        失败.append(kw)
                            else:
                                for kw in results:
                                    成功.append(kw) if kw['msg'] is True else 失败.append(kw)
                            if type(返值分组尺寸) is int:
                                len_成功 = len(成功)
                                if len_成功 >= 返值分组尺寸:
                                    print(f'\r本次执行成功共 {len_成功} 条', end='    ')
                                    yield 成功
                                    成功 = []
                            time.sleep(重试间隔)
                if 成功 or (type(返值分组尺寸) is not int):
                    print(f'\r本次执行成功共 {len(成功)} 条', end='    ')
                    yield 成功
                print('\r执行结束', end='    ')
                yield '执行结束'
                print(f'\r执行失败共 {len(失败)} 条', end='    ')
                yield 失败

            return newfunc
        
        return 装饰器

# 开始测试
# 开始测试


# -!- coding: utf-8 -!-
import requests, re, json, time
from scrapy import Selector
from copy import deepcopy
# 同文件夹模块
from .req_base import req_base


class req_baidu_pc(req_base):

    rn = 10
    headers = {
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
        'Accept-Encoding': 'gzip, deflate, br',
        'Accept-Language': 'zh-CN,zh;q=0.9',
        'Cache-Control': 'max-age=0',
        'Connection': 'close',
        'Upgrade-Insecure-Requests': '1',
        'Host': 'www.baidu.com',
    }
    url_get_title = None

    def __init__(self):
        self.headers = deepcopy(self.headers)  # 补丁
        req_base.__init__(self)

    def 搜索(self, kw, rn=None, **kvs):
        url = f'https://www.baidu.com/s?wd={kw}&rn={rn or self.rn}&sl_lang=cn&rsv_srlang=cn&rsv_rq=cn&ct=0&tfflag=0&gpc=stf%3D'
        return self.get(url=url, **kvs)
    
    def json搜索(self, kw, rn=None, **kvs):
        url = f'https://www.baidu.com/s?ie=utf8&oe=utf8&wd={kw}&tn=json&ch=9&rn={rn or self.rn}'
        return self.get(url=url, **kvs)

    def kw_get_num(self, kw, rn=10):  # 查收录, rn 只需要10即可
        r = self.搜索(kw=kw, rn=rn)
        html = self.r_get_html(r=r)
        return self.html_get_num(html=html)
    
    # 输入html, 返回收录量
    def html_get_num(self, html):
        if re.findall('没有找到该URL。您可以直接访问.+还可.+提交网址.+给我们.+以下是网页中包含.+的结果', html, re.S):
            return 0
        
        for text in ('百度为您找到相关结果数?约(.*)个', '找到相关结果数?约(.*)个'):
            try:
                num = re.findall(text, html)
                return int(''.join(re.findall('\d', num[0])))
            except: pass
        
        try:  # 该网站共有(.+)个网页被百度收录
            num = re.findall('该网站共有(.+)个网页被百度收录', html, re.S)
            return int(''.join(re.findall('\d', num[0])))
        except: pass

        if re.findall('很?抱歉.*没有找到与.+相关的网页', html, re.S):
            return 0
        raise Exception("查收录量失败")

    # 传入html, 传出以排名为key的字典
    def html_get_results(self, html):
        rankings = {}
        tags = Selector(text=html).xpath("//div[contains(@class,'result c-container')]")
        for tag in tags:
            try:
                # 排名作为key
                rankings[int(tag.attrib['id'])] = item = {}
                x = tag.xpath(".//div[re:test(@class,'f13.+se_st_footer')]/a[@target='_blank']")[0]
                # 提取顶部链接
                domain_href = x.xpath("./@href").extract()[0]
                顶部文字 = tag.xpath(".//h3[re:test(@class,'t')]/a[@target='_blank']")[0]
                item['顶部文字'] = 顶部文字.xpath('string(.)').extract()[0]
                item['顶部链接'] = re.sub('\.{3}$', '', re.sub('\xa0$', '', domain_href))
                # 提取底部文字
                try:  # A型html模板的熊掌号
                    item['底部文字'] = x.xpath("./span[@class='nor-src-wrap']")[0].xpath('string(.)').extract()[0]
                except:
                    if x.xpath("b"):
                        item['底部文字'] = x.xpath('string(.)').extract()[0]  # 域名的文本
                    else:
                        item['底部文字'] = x.xpath("./text()").extract()[0]
            except:
                pass
        return rankings
